# Copyright 2021 IBM All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This file defines the constants used by the SDK.
"""

DEFAULT_SEGMENT_ID = '$$null$$'
DEFAULT_ENTITY_ID = '$$null$$'
DEFAULT_HTTP_TYPE = 'https://'
DEFAULT_WSS_TYPE = "wss://"
DEFAULT_BASE_URL = ".apprapp.cloud.ibm.com"
IAM_TEST_URL = "https://iam.test.cloud.ibm.com/identity/token"
IAM_PROD_URL = "https://iam.cloud.ibm.com/identity/token"
SDK_NAME = "appconfiguration-python-sdk"
